import logging


def create_console_handler():
    console_handler = logging.StreamHandler()
    console_formatter = logging.Formatter("%(asctime)s %(name)s %(levelname)s: %(message)s")
    console_handler.setFormatter(console_formatter)
    return console_handler


def get_logger_name(clazz):
    return "{}.{}".format(clazz.__module__, clazz.__name__)



import pytest
from assertpy import assert_that

from klimalogger.calc import TemperatureCalc


class TestCalcTemperature:

    @pytest.fixture
    def uut(self):
        return TemperatureCalc()

    @pytest.mark.parametrize("temperature,relative_humidity,expected", [
        (20, 80, 16.444),
        (25, 10, -8.75),
        (50, 90, 47.9)
    ])
    def test_dew_point(self, uut, temperature, relative_humidity, expected):
        result = uut.dew_point(temperature, relative_humidity)

        assert_that(result).is_close_to(expected, 0.01)



from assertpy import assert_that
from mock import Mock, PropertyMock

import klimalogger


class TestDataBuilderTest:

    def setup_method(self):
        self.config = Mock()
        type(self.config).client_location_name = PropertyMock(return_value='<location>')
        type(self.config).client_host_name = PropertyMock(return_value='<host>')
        self.uut = klimalogger.DataBuilder(self.config)

    def test_data_is_empty_by_default(self):
        assert_that(self.uut.data).is_empty()

    def test_create_entry(self):
        self.uut.add('<sensor>', '<type>', '<unit>', '<value>', is_calculated=False)

        assert_that(self.uut.data).contains({
            'measurement': 'data',
            'tags': {'host': '<host>', 'location': '<location>', 'type': '<type>', 'unit': '<unit>',
                     'sensor': '<sensor>', 'calculated': False},
            'time': self.uut.timestamp,
            'fields': {'value': '<value>'}
        })



import configparser
import logging
import socket
from pathlib import Path

from injector import singleton, provider, Module, inject
from lazy import lazy

log = logging.getLogger(__name__)


@singleton
class Config:
    @inject
    def __init__(self, config_parser: configparser.ConfigParser):
        self.config_parser = config_parser

    @lazy
    def client_location_name(self) -> str:
        return self.config_parser.get('client', 'location_name')

    @lazy
    def client_host_name(self) -> str:
        return socket.gethostname()

    @lazy
    def store_type(self) -> str:
        return self.config_parser.get('store', 'type', fallback='direct')

    @lazy
    def store_org(self) -> str:
        return self.config_parser.get('store', 'org', fallback=None)

    @lazy
    def service_username(self) -> str:
        return self.config_parser.get('store', 'username')

    @lazy
    def service_password(self) -> str:
        return self.config_parser.get('store', 'password')

    @lazy
    def service_name(self) -> str:
        return self.config_parser.get('store', 'name')

    @lazy
    def service_host(self) -> str:
        return self.config_parser.get('store', 'host')

    @lazy
    def store_url(self) -> str:
        return self.config_parser.get('store', 'url')

    @lazy
    def service_port(self) -> int:
        port_string = self.config_parser.get('store', 'port')
        return int(port_string)

    @lazy
    def queue_username(self) -> str:
        return self.config_parser.get('queue', 'username')

    @lazy
    def queue_qos(self) -> int:
        qos_string = self.config_parser.get('store', 'queue_qos', fallback="1")
        return int(qos_string)

    @lazy
    def queue_prefix(self) -> str:
        return self.config_parser.get('store', 'queue_prefix', fallback='sensors')

    @lazy
    def queue_virtual_host(self) -> str:
        return self.config_parser.get('queue', 'virtual_host')

    @lazy
    def log_path(self) -> str:
        return self.config_parser.get('log', 'path')


class ConfigModule(Module):
    @singleton
    @provider
    def provide_config_parser(self) -> configparser.ConfigParser:
        etc = Path("/etc")
        config_filename = "klimalogger.conf"

        config_file_locations = [Path(config_filename), etc / "klimalogger" / config_filename, etc / config_filename]

        for config_file_location in config_file_locations:
            if config_file_location.exists():
                log.info("reading config file location %s", config_file_location)
                config_parser = configparser.ConfigParser()
                config_parser.read(config_file_location)
                return config_parser

        raise IOError("config file not found")



import logging
import typing
from collections.abc import Iterable
from typing import Union, List

from influxdb_client import InfluxDBClient, Point
from influxdb_client.client.exceptions import InfluxDBError
from injector import inject

from .client import StoreClient
from ..config import Config

log = logging.getLogger(__name__)


class BatchingCallback:
    def error(self, conf: (str, str, str), data: str, exception: InfluxDBError):
        print(f"Cannot write batch: {conf}, data: {data} due: {exception}")

    def retry(self, conf: (str, str, str), data: str, exception: InfluxDBError):
        print(f"Retryable error occurs for batch: {conf}, data: {data} retry: {exception}")


class InfluxDb2Store(StoreClient):
    @inject
    def __init__(self, configuration: Config):
        try:
            self.client = InfluxDBClient(
                url=configuration.store_url,
                org=configuration.store_org,
                username=configuration.service_username,
                password=configuration.service_password,
            )
        except Exception as e:
            log.error("could not create client", exc_info=e)
            self.client = None
        self.name = configuration.service_name
        self.callbacks = BatchingCallback()
        assert self.client.ping()

    def store(self, data: Union[dict, typing.Iterable[dict]]):
        if self.client:
            assert self.client.ping()

            log.info("write data")
            if isinstance(data, Iterable):
                record = [Point.from_dict(entry) for entry in data]
            else:
                record = Point.from_dict(data)
            with self.client.write_api(
                    error_callback=self.callbacks.error,
                    retry_callback=self.callbacks.retry) as write_api:
                write_api.write(self.name, record=record)
        else:
            log.error("client not available")
            raise RuntimeError("client not available")



from functools import partial

import mock
import pytest

from klimalogger import MeasurementDispatcher, DataBuilder
from klimalogger.data_builder import DataBuilderFactory
from klimalogger.measurement import SensorFactory, Measurements


class TestSensorFactory:

    @pytest.fixture
    def bus_i2c(self):
        with mock.patch("busio.I2C") as bus_i2c:
            yield bus_i2c()

    @pytest.fixture
    def configuration(self):
        with mock.patch("configparser.ConfigParser") as config_parser:
            yield config_parser()

    def test_sorting(self, configuration):
        calls = []

        def register_call(*args, **kwargs):
            calls.append((kwargs.pop('caller', None), mock.call(*args, **kwargs)))
            return mock.DEFAULT

        configuration.get.return_value = "foo,bar,baz"
        foo_sensor = mock.Mock()
        foo_sensor.name = "foo"
        foo_sensor.priority = 2
        foo_sensor.measure.side_effect = partial(register_call, caller="foo")
        bar_sensor = mock.Mock()
        bar_sensor.name = "bar"
        bar_sensor.priority = 3
        bar_sensor.measure.side_effect = partial(register_call, caller="bar")
        baz_sensor = mock.Mock()
        baz_sensor.name = "baz"
        baz_sensor.priority = 1
        baz_sensor.measure.side_effect = partial(register_call, caller="baz")

        sensor_factory = mock.Mock(SensorFactory)

        def load_module(sensor_name):
            return {
                "foo": foo_sensor,
                "bar": bar_sensor,
                "baz": baz_sensor,
            }[sensor_name]

        sensor_factory.create_sensor.side_effect = load_module



        data_builder_factory = mock.Mock(spec=DataBuilderFactory)
        uut = MeasurementDispatcher(configuration, sensor_factory, data_builder_factory)

        uut.measure()

        assert len(calls) == 3
        assert calls[0] == ("baz", mock.call(data_builder_factory.get(), Measurements()))
        assert calls[1] == ("foo", mock.call(data_builder_factory.get(), Measurements()))
        assert calls[2] == ("bar", mock.call(data_builder_factory.get(), Measurements()))



# -*- coding: utf8 -*-
import busio
from injector import singleton, inject

from .. import DataBuilder
from ..calc import PressureCalc
from ..measurement import Measurements

try:
    import configparser
except ImportError:
    import configparser as configparser

import adafruit_bmp3xx


@singleton
class Sensor:
    name = "BMP3xx"
    priority = 2

    @inject
    def __init__(self, i2c_bus: busio.I2C, config_parser: configparser.ConfigParser, pressure_calc: PressureCalc):
        self.pressure_calc = pressure_calc
        self.elevation = int(config_parser.get('bmp3xx_sensor', 'elevation'))

        self.driver = adafruit_bmp3xx.BMP3XX_I2C(i2c_bus)

    def measure(self, data_builder: DataBuilder, measurements: Measurements) -> None:
        temperature = self.driver.temperature
        pressure = self.driver.pressure
        sea_level_pressure = self.pressure_calc.sea_level_pressure(pressure, temperature, self.elevation)

        data_builder.add(self.name, "pressure", "hPa", round(pressure, 2))
        data_builder.add(self.name, "sea level pressure", "hPa", round(sea_level_pressure, 2), is_calculated=True)



class PressureCalc:

    def sea_level_pressure(self, pressure: float, temperature: float, elevation: float) -> float:
        return pressure * pow(1 - (0.0065 * elevation / (temperature + 0.0065 * elevation + 273.15)), -5.255)



import datetime
import time

from injector import singleton, inject

from .config import Config


class DataBuilder:
    def __init__(self, configuration: Config):
        self.location = configuration.client_location_name
        self.host_name = configuration.client_host_name
        self.timestamp = time.time()
        self.data = []

    def add(self, sensor: str, measurement_type: str, measurement_unit: str, measurement_value: float,
            is_calculated: bool = False):
        if measurement_value is not None:
            self.data += [self.create(sensor, measurement_type, measurement_unit, measurement_value, is_calculated)]

    def create(self, sensor: str, measurement_type: str, measurement_unit: str, measurement_value: float,
               is_calculated: bool = False):
        return {
            "measurement": "data",
            "tags": {
                "host": self.host_name,
                "location": self.location,
                "type": measurement_type,
                "unit": measurement_unit,
                "sensor": sensor,
                "calculated": is_calculated
            },
            "time": self.timestamp,
            "fields": {
                "value": measurement_value
            }
        }


@singleton
class DataBuilderFactory:

    @inject
    def __init__(self, config: Config):
        self.config = config

    def get(self) -> DataBuilder:
        return DataBuilder(self.config)



#!/usr/bin/env python3
# -*- coding: utf8 -*-

import argparse
import logging

import klimalogger

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--verbose", dest="verbose", action="store_true", help="verbose output")
    parser.add_argument("-d", "--debug", dest="debug", action="store_true", help="debug output")
    parser.add_argument("--version", help="print package version", action="store_true")
    parser.add_argument("--check", help="measure once and print the results", action="store_true")
    parser.add_argument("--service", help="run as service performing periodic measurements", action="store_true")
    args = parser.parse_args()

    klimalogger.add_log_handler(klimalogger.logger.create_console_handler())
    klimalogger.set_log_level(logging.WARN)
    if args.debug:
        klimalogger.set_log_level(logging.DEBUG)
    elif args.verbose:
        klimalogger.set_log_level(logging.INFO)

    client = klimalogger.client()
    try:
        if args.version:
            import pkg_resources
            package_version = pkg_resources.get_distribution('klimalogger').version
            print(f"Version {package_version}")
        if args.check:
            print(client.measure())
        else:
            if args.service:
                client.measure_and_store_periodically()
            else:
                client.measure_and_store()
    except Exception as e:
        print("Error", e)
        return 10
    return 0

if __name__ == "__main__":
    main()



import logging

from influxdb import InfluxDBClient
from injector import inject

from .client import StoreClient
from ..config import Config

log = logging.getLogger(__name__)


class InfluxDbStore(StoreClient):
    @inject
    def __init__(self, configuration: Config):
        try:
            self.client = InfluxDBClient(
                host=configuration.service_host,
                port=configuration.service_port,
                database=configuration.service_name,
                username=configuration.service_username,
                password=configuration.service_password,
                timeout=5)
        except Exception as e:
            log.error("could not create client", e)
            self.client = None

    def store(self, data: dict):
        if self.client:
            log.info("write data")
            self.client.write_points(data)
        else:
            log.error("client not available")
            raise RuntimeError("client not available")



# -*- coding: utf8 -*-
import logging

import adafruit_sht4x
import busio
from injector import singleton, inject

from .. import DataBuilder
from ..calc import TemperatureCalc
from ..measurement import Measurements

log = logging.getLogger(__name__)

@singleton
class Sensor:
    name = "SHT4x"
    priority = 1

    @inject
    def __init__(self, i2c_bus: busio.I2C, temperature_calc: TemperatureCalc):
        log.info("init")
        self.temperature_calc = temperature_calc

        self.driver = adafruit_sht4x.SHT4x(i2c_bus)

        # self.sensor.mode = adafruit_sht4x.Mode.NOHEAT_HIGHPRECISION
        # Can also set the mode to enable heater
        # sht.mode = adafruit_sht4x.Mode.LOWHEAT_100MS

    def measure(self, data_builder: DataBuilder, measurements: Measurements) -> None:
        (temperature, relative_humidity) = self.driver.measurements

        if temperature > -40.0:
            try:
                dew_point = self.temperature_calc.dew_point(temperature, relative_humidity)
                dew_point = round(dew_point, 2)
            except ValueError:
                dew_point = None

            temperature = round(temperature, 2)
            relative_humidity = round(relative_humidity, 2)
        else:
            temperature = None
            relative_humidity = None
            dew_point = None

        if temperature and relative_humidity and dew_point and -30 < temperature < 80 and 5 < relative_humidity <= 100:
            measurements.temperature = temperature
            measurements.relative_humidity = relative_humidity

            data_builder.add(self.name, "temperature", "°C", temperature)
            if dew_point:
                data_builder.add(self.name, "dew point", "°C", dew_point, True)
            data_builder.add(self.name, "relative humidity", "%", relative_humidity)



import pytest
from assertpy import assert_that

from klimalogger.calc import PressureCalc


class TestCalcTemperature:

    @pytest.fixture
    def uut(self):
        return PressureCalc()

    def test_sea_level_value(self, uut):
        result = uut.sea_level_pressure(1000, 20, 0)

        assert_that(result).is_close_to(1000, 0.01)

    def test_elevation_value(self, uut):
        result = uut.sea_level_pressure(943.71, 20, 500)

        assert_that(result).is_close_to(1000, 0.01)



import logging

from injector import Module, provider, singleton

from .client import StoreClient
from .queue import QueueStore
from ..config import Config

log = logging.getLogger(__name__)

def influxdb_store_factory(config: Config):
    if config.store_org is not None:
        from .influxdb2 import InfluxDb2Store
        return InfluxDb2Store(config)
    else:
        from .influxdb import InfluxDbStore
        return InfluxDbStore(config)

class StoreModule(Module):
    @provider
    @singleton
    def store_provider(self, config: Config) -> StoreClient:
        log.info("store provider type: %s", config.store_type)

        if config.store_type == 'queue':
            return QueueStore(config)
        else:
            return influxdb_store_factory(config)
